<?php

/*
Plugin Name: Wordpress.com Stats Helper
Plugin URI: http://vlad.bailescu.ro/wordpress/plugin-stats-helper-functions-and-widgets/
Description: This plugin helps you extract data from wordpress.com stats and use it across your blog.
Version: 0.1
Author: Vlad Bailescu
Author URI: http://vlad.bailescu.ro/

CHANGES
2008.02.23 - Version 0.1, Most visited posts function and widget
*/

require_once(ABSPATH . 'wp-includes/class-snoopy.php');

// Splits a string into an array of tokens, delimited by delimiter char
// tokens in input string containing the delimiter character or the literal escape character are surrounded by a pair of escape characteres
// a literal escape character is produced by the escape character appearing twice in sequence
// default delimiter character and escape character are suitable for Excel-exported CSV formatted lines
// Source: http://www.php.net/manual/en/function.explode.php#75876
function splitWithEscape ($str, $delimiterChar = ',', $escapeChar = '"') {
	$len = strlen($str);
	$tokens = array();
	$i = 0;
	$inEscapeSeq = false;
	$currToken = '';
	while ($i < $len) {
		$c = substr($str, $i, 1);
		if ($inEscapeSeq) {
			if ($c == $escapeChar) {
				// lookahead to see if next character is also an escape char
				if ($i == ($len - 1)) {
					// c is last char, so must be end of escape sequence
					$inEscapeSeq = false;
				} else if (substr($str, $i + 1, 1) == $escapeChar) {
					// append literal escape char
					$currToken .= $escapeChar;
					$i++;
				} else {
					// end of escape sequence
					$inEscapeSeq = false;
				}
			} else {
				$currToken .= $c;
			}
		} else {
			if ($c == $delimiterChar) {
				// end of token, flush it
				array_push($tokens, $currToken);
				$currToken = '';
			} else if ($c == $escapeChar) {
				// begin escape sequence
				$inEscapeSeq = true;
			} else {
				$currToken .= $c;
			}
		}
		$i++;
	}
	// flush the last token
	array_push($tokens, $currToken);
	return $tokens;
}

// Helper function to list most visited posts
function wpcomstats_most_visited_posts($posts = 5, $before = '&raquo;', $after = '<br />') {
	// Check cache
	$rows = wp_cache_get('most_visited_posts_'.$posts);
	if (!$rows) {
		// Check if worpress.com stats plugin is installed
		if (!function_exists('stats_get_api_key') || !function_exists('stats_get_option')) {
			echo 'Wordpress.com stats not installed!';
			return;
		}
		// Check if the API key is defined
		$api_key = stats_get_api_key();
		if (empty($api_key)) {
			echo 'API Key not defined!';
			return;
		}
		// Fetch the corresponding CSV
		$snoopy = new Snoopy();
		if (@$snoopy->fetch('http://stats.wordpress.com/csv.php?'.
				'api_key='.$api_key.
				'&blog_id='.stats_get_option('blog_id').
				'&table=postviews&summarize=true'.
				'&limit='.$posts)) {
			$results = trim(str_replace("\r\n", "\n", $snoopy->results));
			$rows = explode("\n", $results);
			// Cache the results rows
			wp_cache_set('most_visited_posts_'.$posts, $rows);
		}
	}
	for ($i = 1, $n = count($rows); $i < $n; $i++) {
		// Split columns
		$cols = splitWithEscape($rows[$i]);
		$name = $cols[1];
		$url = $cols[2];
		$count = $cols[3];
		if (strlen($name) > 40) {
			// Shorten long post names to max 40 chars
			$short_name = substr($name, 0, 40).'...';
		} else {
			$short_name = $name;
		}
		$most_visited_posts .= $before.'<a href="'.$url.'" title="'.$name.'">'.$short_name.'</a> ('.$count.')'.$after;
	}
	echo $most_visited_posts;
}

// Widget initialization
function wpcomstats_helper_init() {

	// Check to see if widgets are permitted
	if ( !function_exists('register_sidebar_widget') || 
			!function_exists('register_widget_control') ) {
		return;
	}

	// Define widget - no magic here
	function widget_wpcomstats_most_visited_posts($args) {
		extract($args);
		$options = get_option('widget_wpcomstats_most_visited_posts'); 
		$title = $options['title'];
		if ($title || $title == '') { $title = 'Most Visited Posts'; }
		$posts = $options['posts'];
		if (!$posts || $posts == '') { $posts = 5; }
		echo $before_widget;
		echo $before_title.$title.$after_title;
		echo '<ul>';
		echo wpcomstats_most_visited_posts($posts, '<li>', '</li>');
		echo '</ul>';
		echo $after_widget;
	}

	function widget_wpcomstats_most_visited_posts_control() {
		$options = $newoption = get_option('widget_wpcomstats_most_visited_posts');
		if ( $_POST['widget_wpcomstats_most_visited_posts-submit'] ) {
			$newoptions['title'] = strip_tags(stripslashes($_POST['widget_wpcomstats_most_visited_posts-title']));
			$newoptions['posts'] = strip_tags(stripslashes($_POST['widget_wpcomstats_most_visited_posts-posts']));
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_wpcomstats_most_visited_posts', $options);
		}
?>
		<label for="widget_wpcomstats_most_visited_posts-title" style="line-height:25px;display:block;">
			<?php _e('Title:', 'widgets'); ?>
			<input type="text" id="widget_wpcomstats_most_visited_posts-title" name="widget_wpcomstats_most_visited_posts-title" value="<?php echo wp_specialchars($options['title'], true); ?>" />
		</label> 
		<label for="widget_wpcomstats_most_visited_posts-posts" style="line-height:25px;display:block;">
			<?php _e('Posts:', 'widgets'); ?>
			<input type="text" id="widget_wpcomstats_most_visited_posts-posts" name="widget_wpcomstats_most_visited_posts-posts" value="<?php echo wp_specialchars($options['posts'], true); ?>" />
		</label> 
		<input type="hidden" name="widget_wpcomstats_most_visited_posts-submit" id="widget_wpcomstats_most_visited_posts-submit" value="1" />
<?php
	}

	// Register widget
	register_sidebar_widget(array('Most Visited Posts', 'widgets'),  'widget_wpcomstats_most_visited_posts'/*[, $title [, $posts]]*/);
	register_widget_control(array('Most Visited Posts', 'widgets'), 'widget_wpcomstats_most_visited_posts_control'); 
}

// Call widget initialization
add_action('widgets_init', 'wpcomstats_helper_init');

?>
